#include "SusieArchiveWrapper.h"
namespace HongliangSoft {
  namespace Utilities {
    SusieArchiveWrapper::SusieArchiveWrapper(HMODULE module) : SusieWrapper(module) {
      getArchiveInfoHandler = (GetArchiveInfoHandler)GetProcAddress(module, "GetArchiveInfo");
      getFileInfoHandler = (GetFileInfoHandler)GetProcAddress(module, "GetFileInfo");
      getFileHandler = (GetFileHandler)GetProcAddress(module, "GetFile");
    }
    void SusieArchiveWrapper::Dispose(bool disposing) {
      if (!this->Disposed && disposing) {
        this->getArchiveInfoHandler = NULL;
        this->getFileInfoHandler = NULL;
        this->getFileHandler = NULL;
      }
      SusieWrapper::Dispose(disposing);
    }
    FileInArchive* SusieArchiveWrapper::GetFilesInArchive(String* archiveFile)[] {
      if (!(this->CanGetArchiveInfo))
        throw new NotSupportedException();
      IntPtr file = Marshal::StringToCoTaskMemAnsi(archiveFile);
      try {
        HLOCAL local;
        this->getArchiveInfoHandler((LPSTR)file.ToPointer(), 0, 0, &local);
        if (local != NULL) {
          ArrayList* infos = new ArrayList();
          try {
            FILEINFO* fileinfo = (FILEINFO*)LocalLock(local);
            while (fileinfo->method[0] != '\0') {
              infos->Add(new FileInArchive(archiveFile, fileinfo));
              fileinfo++;
            }
            return static_cast<FileInArchive*[]>(infos->ToArray(__typeof(FileInArchive)));
          }
          __finally {
            LocalUnlock(local);
            LocalFree(local);
          }
        }
      }
      __finally {
        Marshal::FreeCoTaskMem(file);
      }
      return NULL;
    }
    FileInArchive* SusieArchiveWrapper::GetFileInfo(String* archiveFile, String* fileName, bool caseSensitive) {
      if (!(this->CanGetFileInfo))
        throw new NotSupportedException();
      IntPtr archive = Marshal::StringToCoTaskMemAnsi(archiveFile);
      IntPtr name = Marshal::StringToCoTaskMemAnsi(fileName);
      try {
        FILEINFO info;
        this->getFileInfoHandler((LPSTR)archive.ToPointer(), 0, (LPSTR)name.ToPointer(),
                                 (caseSensitive ? CASE_SENSITIVE : IGNORE_CASE) | FROM_FILE, &info);
        return new FileInArchive(archiveFile, &info);
      }
      __finally {
        Marshal::FreeCoTaskMem(archive);
        Marshal::FreeCoTaskMem(name);
      }
    }
    String* SusieArchiveWrapper::GetFileToFile(FileInArchive* getFile, String* outputDirectory) {
      return this->GetFileToFile(getFile, outputDirectory, SusieWrapper::noProcess);
    }
    String* SusieArchiveWrapper::GetFileToFile(FileInArchive* getFile, String* outputDirectory, ProgressCallback* callback) {
      if (!(this->CanGetFile))
        throw new NotSupportedException();
      IntPtr archive = Marshal::StringToCoTaskMemAnsi(getFile->ArchiveFile);
      IntPtr output = Marshal::StringToCoTaskMemAnsi(outputDirectory);
      try {
        this->getFileHandler((LPSTR)archive.ToPointer(), getFile->Position, output.ToPointer(), FROM_FILE | TO_FILE, callback, 0);
        Char trimmer[] = {'\\'};
        return String::Concat(outputDirectory->TrimEnd(trimmer), S"\\", getFile->FileName);
      }
      __finally {
        Marshal::FreeCoTaskMem(archive);
        Marshal::FreeCoTaskMem(output);
      }
    }
    Stream* SusieArchiveWrapper::GetFileToStream(FileInArchive* getFile) {
      return this->GetFileToStream(getFile, SusieWrapper::noProcess);
    }
    Stream* SusieArchiveWrapper::GetFileToStream(FileInArchive* getFile, ProgressCallback* callback) {
      if (!(this->CanGetFile))
        throw new NotSupportedException();
      if (getFile->IsDirectory)
        return NULL;
      IntPtr archive = Marshal::StringToCoTaskMemAnsi(getFile->ArchiveFile);
      try {
        HLOCAL local;
        if (this->getFileHandler((LPSTR)archive.ToPointer(), getFile->Position, &local, FROM_FILE | TO_MEMORY, callback, 0) == 0) {
          const int size = 0x200000;
          int wrote = 0, last = LocalSize(local);
          Byte buffer[] = new Byte[size];
          IntPtr localPtr = IntPtr(LocalLock(local));
          if (localPtr != IntPtr::Zero) {
            try {
              Stream* stream = new MemoryStream();
              while (last >= size) {
                Marshal::Copy(localPtr, buffer, 0, size);
                last -= size;
                stream->Write(buffer, 0, size);
              }
              Marshal::Copy(localPtr, buffer, 0, last);
              stream->Write(buffer, 0, last);
              stream->Position = 0;
              return stream;
            }
            __finally {
              LocalUnlock(local);
              LocalFree(local);
            }
          }
        }
        else
          return NULL;
      }
      __finally {
        Marshal::FreeCoTaskMem(archive);
      }
    }
  }
}